﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace PaymentMethodSample
{
    public class BulkPayment : NotificationObject, IDataErrorInfo
    {
        public BulkPayment()
        { }

        //public BulkPayment(int? bulkPaymentID, int? payerTypeID, string payerType, 
        //   int? payerID, int? invoiceID, DateTime? paymentDate, int paymentMethodID, PaymentMethod paymentMethod, 
        //    string paymentTrackingNumber

        public BulkPayment(int? bulkPaymentID, int? payerTypeID, string payerType, int? payerID, int? invoiceID, DateTime?
            paymentDate, int bulkpaymentMethodID, PaymentMethod paymentMethod, string paymentTrackingNumber
            , decimal paymentAmount, string bankBatchNumber)
        {
            BulkPaymentID = bulkPaymentID;
            PayerTypeID = payerTypeID;
            PayerType = payerType;
            PayerID = payerID;
            InvoiceID = invoiceID;
            PaymentDate = paymentDate;

            //IG  added to track selected item id in PaymentMethod (sort & group)
            BulkPaymentMethodID = bulkpaymentMethodID;

            PaymentMethod = paymentMethod;
            PaymentTrackingNumber = paymentTrackingNumber;
            PaymentAmount = paymentAmount;
            BankBatchNumber = bankBatchNumber;
        }

        private int? _bulkPaymentID;
        /// <summary>
        /// ID of the bulk payment
        /// </summary>
        public int? BulkPaymentID
        {
            get { return _bulkPaymentID; }
            set
            {
                if (_bulkPaymentID != value)
                {
                    _bulkPaymentID = value;
                    FirePropertyChanged(() => BulkPaymentID);
                }
            }
        }

        private int? _payerTypeID;
        /// <summary>
        /// Payer Type ID
        /// </summary>
        public int? PayerTypeID
        {
            get { return _payerTypeID; }
            set
            {
                if (_payerTypeID != value)
                {
                    _payerTypeID = value;
                    FirePropertyChanged(() => PayerTypeID);
                }
            }
        }

        private string _payerType;
        /// <summary>
        /// Payer Type
        /// </summary>
        public string PayerType
        {
            get { return _payerType; }
            set
            {
                if (_payerType != value)
                {
                    _payerType = value;
                    //ValidateAll();
                    FirePropertyChanged(() => PayerType);
                }
            }
        }

        private int? _payerID;
        /// <summary>
        /// Payer ID
        /// </summary>
        public int? PayerID
        {
            get { return _payerID; }
            set
            {
                if (_payerID != value)
                {
                    _payerID = value;
                    // ValidateAll();
                    FirePropertyChanged(() => PayerID);
                }
            }
        }

        private int? _invoiceID;
        /// <summary>
        /// Invoice ID
        /// </summary>
        public int? InvoiceID
        {
            get { return _invoiceID; }
            set
            {
                if (_invoiceID != value)
                {
                    _invoiceID = value;
                    FirePropertyChanged(() => InvoiceID);
                }
            }
        }

        private DateTime? _paymentDate;
        /// <summary>
        /// Payment Date
        /// </summary>
        public DateTime? PaymentDate
        {
            get { return _paymentDate; }
            set
            {
                if (_paymentDate != value)
                {
                    _paymentDate = value;
                    FirePropertyChanged(() => PaymentDate);
                }
            }
        }

        //IG  added to track selected item id in PaymentMethod (sort & group)
        private int _blkpaymentMethodID;
        /// <summary>
        /// Payment Method
        /// </summary>
        public int BulkPaymentMethodID
        {
            get { return _blkpaymentMethodID; }
            set
            {
                if (_blkpaymentMethodID != value)
                {
                    _blkpaymentMethodID = value;
                    FirePropertyChanged(() => BulkPaymentMethodID);
                }
            }
        }


        private PaymentMethod _paymentMethod;
        /// <summary>
        /// Payment Method
        /// </summary>
        public PaymentMethod PaymentMethod
        {
            get { return _paymentMethod; }
            set
            {
                if (_paymentMethod != value)
                {
                    _paymentMethod = value;
                    FirePropertyChanged(() => PaymentMethod);
                }
            }
        }

        private string _paymentTrackingNumber;
        /// <summary>
        /// Payment Tracking Number
        /// </summary>
        public string PaymentTrackingNumber
        {
            get { return _paymentTrackingNumber; }
            set
            {
                if (_paymentTrackingNumber != value)
                {
                    _paymentTrackingNumber = value;
                    //ValidateAll();
                    FirePropertyChanged(() => PaymentTrackingNumber);
                }
            }
        }

        private decimal _paymentAmount;
        /// <summary>
        /// Payment Amount
        /// </summary>
        public decimal PaymentAmount
        {
            get { return _paymentAmount; }
            set
            {
                if (_paymentAmount != value)
                {
                    _paymentAmount = value;
                    //ValidateAll();
                    FirePropertyChanged(() => PaymentAmount);
                }
            }
        }

        private string _bankBatchNumber;
        /// <summary>
        /// Bank Batch Number
        /// </summary>
        public string BankBatchNumber
        {
            get { return _bankBatchNumber; }
            set
            {
                if (_bankBatchNumber != value)
                {
                    _bankBatchNumber = value;
                    FirePropertyChanged(() => BankBatchNumber);
                }
            }
        }

        private bool _isDirty;
        /// <summary>
        /// Bool tells whether item has changed or not
        /// </summary>
        public bool IsDirty
        {
            get { return _isDirty; }
            set
            {
                if (_isDirty != value)
                {
                    _isDirty = value;
                    FirePropertyChanged(() => IsDirty);
                }
            }
        }

        /// <summary>
        /// All properties that should be validated
        /// </summary>
        public readonly string[] ValidatedProperties = { "PayerType", "PayerID", "InvoiceID", "PaymentMethod", "PaymentTrackingNumber", "PaymentAmount" };

        /// <summary>
        /// Bool tells whether item contains an error
        /// </summary>
        public bool IsValid
        {
            get
            {
                foreach (string property in ValidatedProperties)
                {
                    if (GetValidationError(property) != null) //there is an error
                        return false;
                }

                return true;
            }
        }

        private string GetValidationError(string propertyName)
        {
            string error = null;

            switch (propertyName)
            {
                case "PayerType":
                    {
                        if (String.IsNullOrWhiteSpace(PayerType))
                            error = "Payer Type cannot be empty";
                        break;
                    }
                case "PayerID":
                    {
                        if (PayerID == null)
                            error = "Payer ID cannot be empty";
                        break;
                    }
                case "InvoiceID":
                    {
                        if (InvoiceID == null)
                            error = "Invoice # cannot be empty";
                        break;

                    }
                case "PaymentMethod":
                    {
                        if (PaymentMethod == null)
                            error = "Payment Method cannot be empty";
                        break;
                    }
                case "PaymentTrackingNumber":
                    {
                        if (String.IsNullOrWhiteSpace(PaymentTrackingNumber))
                            error = "Tracking Number cannot be empty";
                        break;
                    }
                case "PaymentAmount":
                    {
                        if (PaymentAmount <= 0.00m)
                            error = "Payment Amount must be greater than $0.00";
                        break;
                    }
                //case "BankBatchNumber":
                //    {
                //        if (String.IsNullOrWhiteSpace(BankBatchNumber))
                //            error = "Bank Batch Number From cannot be empty";
                //        break;
                //    }
                default:
                    {
                        error = null;
                        break;
                    }
            }

            return error;
        }

        /// <summary>
        /// Error message for validation
        /// </summary>
        /// <param name="columnName">Field name to be validated</param>
        /// <returns>error message</returns>
        public string this[string columnName]
        {
            get
            {
                string error = GetValidationError(columnName);

                return error;
            }
        }

        public string Error
        {
            get { return null; }
        }
    }
}