﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace PaymentMethodSample
{
    public class PaymentMethod : NotificationObject
    {
        public PaymentMethod()
        { }

        public PaymentMethod(int id, string description, bool active)
        {
            PaymentMethodID = id;
            PaymentMethodDescription = description;
            Active = active;
        }

        private int _paymentMethodID;
        /// <summary>
        /// ID of the Payment Method
        /// </summary>
        public int PaymentMethodID
        {
            get { return _paymentMethodID; }
            set
            {
                if (_paymentMethodID != value)
                {
                    _paymentMethodID = value;
                    FirePropertyChanged(() => PaymentMethodID);
                }
            }
        }
        private string _paymentMethodDescription;
        /// <summary>
        /// Payment Method description
        /// </summary>
        public string PaymentMethodDescription
        {
            get { return _paymentMethodDescription; }
            set
            {
                if (_paymentMethodDescription != value)
                {
                    _paymentMethodDescription = value;
                    FirePropertyChanged(() => PaymentMethodDescription);
                }
            }
        }

        private bool _active;
        public bool Active
        {
            get { return _active; }
            set
            {
                if (_active != value)
                {
                    _active = value;
                    FirePropertyChanged(() => Active);
                }
            }
        }

        /// <summary>
        /// Overrides the Equals function so that don't have to be the exact same instance to be equal
        /// </summary>
        /// <param name="obj"></param>
        /// <returns></returns>
        public override bool Equals(object obj)
        {
            if (obj == null || !(obj is PaymentMethod))
                return false;
            return ((PaymentMethod)obj).PaymentMethodID == this.PaymentMethodID;
        }

        public static bool operator ==(PaymentMethod a, PaymentMethod b)
        {
            // If both are null, or both are same instance, return true.
            if (System.Object.ReferenceEquals(a, b))
            {
                return true;
            }

            // If one is null, but not both, return false.
            if (((object)a == null) || ((object)b == null))
            {
                return false;
            }

            // Return true if the fields match:
            return a.PaymentMethodID == b.PaymentMethodID;
        }

        public static bool operator !=(PaymentMethod a, PaymentMethod b)
        {
            return !(a == b);
        }

        /// <summary>
        /// Included to not violate equals objects must have same hash code
        /// </summary>
        /// <returns></returns>
        public override int GetHashCode()
        {
            return this.PaymentMethodID.GetHashCode();
        }
    }
}
